@@ -60,6 +60,8 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Arabic (Lebanon)"), "ar_LB", NULL, 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Arabic (Saudi Arabia)"), "ar_SA", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Armenian"), "hy_AM", "Հայերեն", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Azerbaijani"), "az_AZ", "Azərbaycanca", 0 },
@@ -94,6 +96,8 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Dutch"), "nl_NL", "Nederlands", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Dutch (Belgium)"), "nl_BE", "Nederlands (België)", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (USA)"), "en_US", "American English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (Australia)"), "en_AU", "Australian English", 0 },
@@ -108,6 +112,10 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (South Africa)"), "en_ZA", "South African English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|English (Malta)"), "en_MT", "Maltese English", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|English (New Zealand)"), "en_NZ", "New Zealand English", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Estonian"), "et_EE", "Eesti", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Finnish"), "fi_FI", "Suomi", 0 },
@@ -116,6 +124,10 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|French (Belgium)"), "fr_BE", "Français (Belgique)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|French (Canada)"), "fr_CA", "Français (Canada)", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|French (Luxembourg)"), "fr_LU", "Français (Luxembourg)", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|French (Switzerland)"), "fr_CH", "Français (Suisse)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Galician"), "gl_ES", "Galego", 0 },
@@ -124,10 +136,14 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|German (Austria)"), "de_AT", "Deutsch (Österreich)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|German (Luxembourg)"), "de_LU", "Deutsch (Luxemburg)", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|German (Switzerland)"), "de_CH", "Deutsch (Schweiz)", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Greek"), "el_GR", "Ελληνικά", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Greek (Cyprus)"), "el_CY", "Ελληνικά (Κύπρος)", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Gujarati"), "gu_IN", "ગુજરાતી", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Hebrew"), "he_IL", "עברית", 0 },
@@ -152,23 +168,25 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Kannada"), "kn_IN", "﻿ಕನ್ನಡ", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-        /*Note translate the A-M to the A-M you used in the group label */
-        { N_("A-M|Kinyarwanda"), "rw_RW", "Kinyarwanda", 0 },
+	{ N_("A-M|Kinyarwanda"), "rw_RW", "Kinyarwanda", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Korean"), "ko_KR", "한국어", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Latvian"), "lv_LV", "Latviešu", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Lithuanian"), "lt_LT", "Lietuvių", 0 },
-        /*Note translate the A-M to the A-M you used in the group label */
-        { N_("A-M|Macedonian"), "mk_MK", NULL, 0 },
-        /*Note translate the A-M to the A-M you used in the group label */
-        { N_("A-M|Malay"), "ms_MY", "Bahasa Melayu", 0 },
-        /*Note translate the A-M to the A-M you used in the group label */
-        { N_("A-M|Malayalam"), "ml_IN", "മലയാളം", 0 },
-        /*Note translate the A-M to the A-M you used in the group label */
-        { N_("A-M|Marathi"), "mr_IN", NULL, 0 },
-        /*Note translate the A-M to the A-M you used in the group label */
-        { N_("A-M|Mongolian"), "mn_MN", "Монгол", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Macedonian"), "mk_MK", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Malay"), "ms_MY", "Bahasa Melayu", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Malayalam"), "ml_IN", "മലയാളം", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Maltese"), "mt_MT", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Marathi"), "mr_IN", NULL, 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Mongolian"), "mn_MN", "Монгол", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Northern Sotho"), "nso_ZA", "Sesotho sa Leboa", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
@@ -178,7 +196,7 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Oriya"), "or_IN", NULL, 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
-	{ N_("N-Z|Panjabi"), "pa_IN", NULL, 0 },
+	{ N_("N-Z|Punjabi"), "pa_IN", NULL, 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Persian"), "fa_IR", "فارسی", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
@@ -194,19 +212,53 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Serbian"), "sr_YU", "српски", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
-	{ N_("N-Z|Serbian"), "sr_CS", "српски", 0 },
+	{ N_("N-Z|Serbian (Serbia & Montenegro)"), "sr_CS", "српски", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian (Montenegro)"), "sr_ME", "српски", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian (Serbia)"), "sr_RS", "српски", 0 },	
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Serbian (Latin)"), "sr_CS@Latn", "srpski (latinica)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Serbian (Jekavian)"), "sr_CS@ije", "српски (ијекавски)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian (Bosnia)"), "sh_BA", "srpski (Bosna i Hercegovina)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Slovak"), "sk_SK", "Slovenský", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Slovenian"), "sl_SI", "Slovenščina", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Spanish"), "es_ES", "Español", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
-	{ N_("N-Z|Spanish (Mexico)"), "es_MX", "Español (Mexico)", 0 },
+	{ N_("N-Z|Spanish (Argentina)"), "es_AR", "Español (Argentina)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Bolivia)"), "es_BO", "Español (Bolivia)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Chile)"), "es_CL", "Español (Chile)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Colombia)"), "es_CO", "Español (Colombia)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Costa Rica)"), "es_CR", "Español (Costa Rica)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Ecuador)"), "es_EC", "Español (Ecuador)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Guatemala)"), "es_GT", "Español (Guatemala)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Mexico)"), "es_MX", "Español (México)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Nicaragua)"), "es_NI", "Español (Nicaragua)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Panama)"), "es_PA", "Español (Panamá)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Peru)"), "es_PE", "Español (Perú)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Paraguay)"), "es_PY", "Español (Paraguay)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (El Salvador)"), "es_SV", "Español (El Salvador)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Uruguay)"), "es_UY", "Español (Uruguay)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Spanish (Venezuela)"), "es_VE", "Español (Venezuela)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Swedish"), "sv_SE", "Svenska", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
@@ -228,9 +280,11 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Welsh"), "cy_GB", "Cymraeg", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Xhosa"), "xh_ZA", "isiXhosa", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Yiddish"), "yi", "ייִדיש", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
-	{ N_("N-Z|Zulu"), "zu_ZA", "IsiZulu", 0 },
+	{ N_("N-Z|Zulu"), "zu_ZA", "isiZulu", 0 },
 	/* This is the POSIX/C locale for english, should really be in Other */
 	{ N_("Other|POSIX/C English"), "C", "POSIX/C English", 0 },
 	{ NULL, NULL }
@@ -247,7 +301,7 @@ gdm_lang_init (void)
 
 	lang_names = g_hash_table_new (g_str_hash, g_str_equal);
 
-	for (i = 0; languages[i].name != NULL; i ++) {
+	for (i = 0; languages[i].name != NULL; i++) {
 		g_hash_table_insert (lang_names,
 				     languages[i].code,
 				     &languages[i]);
@@ -387,7 +441,9 @@ gdm_lang_name (const char *language,
 	if (no_group) {
 		char *p = strchr (name, '|');
 		if (p != NULL) {
-			strcpy (name, p+1);
+			p = g_strdup (p + 1);
+			g_free (name);
+			name = p;
 		}
 	}
 
@@ -442,7 +498,7 @@ lang_collate (gconstpointer a, gconstpointer b)
 		const char *name;
 		name = strchr (_(l1->name), '|');
 		if (name != NULL)
-			name ++;
+			name++;
 		else
 			name = _(l1->name);
 		l1->collate_key = g_utf8_collate_key (name, -1);
@@ -452,7 +508,7 @@ lang_collate (gconstpointer a, gconstpointer b)
 		const char *name;
 		name = strchr (_(l2->name), '|');
 		if (name != NULL)
-			name ++;
+			name++;
 		else
 			name = _(l2->name);
 		l2->collate_key = g_utf8_collate_key (name, -1);
